@@ -12,9 +12,13 @@ from cpggen.logger import DEBUG, LOG, console
 from cpggen.utils import (
     check_command,
     find_csharp_artifacts,
+    find_files,
     find_go_mods,
+    find_gradle_files,
     find_java_artifacts,
     find_makefiles,
+    find_pom_files,
+    find_sbt_files,
 )
 
 runtimeValues = {}
@@ -61,6 +65,8 @@ cpg_tools_map = {
     "jsp": "java -Xmx%(memory)s -jar /usr/local/bin/java2cpg.jar -nb --experimental-langs scala -su -o %(cpg_out)s %(uber_jar)s",
     "jsp-with-blocklist": "java -Xmx%(memory)s -jar /usr/local/bin/java2cpg.jar --experimental-langs scala -su -o %(cpg_out)s %(uber_jar)s",
     "sbom": "cdxgen -r -t %(tool_lang)s -o %(sbom_out)s %(src)s",
+    "export": "joern-export --repr=%(export_repr)s --format=%(export_format)s --out %(cpg_out)s %(src)s",
+    "qwiet": "sl analyze --tag app.group=%(group)s --app %(app)s-%(language)s --%(language)s --cpgupload --bomupload %(sbom)s %(cpg)s",
 }
 
 build_tools_map = {
@@ -72,10 +78,20 @@ build_tools_map = {
             "package",
             "-Dmaven.test.skip=true",
         ],
-        "gradle": [get("GRADLE_CMD", "gradle"), "build"],
+        "gradle": [get("GRADLE_CMD", "%(gradle_cmd)s"), "build"],
         "sbt": ["sbt", "compile"],
     },
-    "android": {"gradle": [get("GRADLE_CMD", "gradle"), "compileDebugSources"]},
+    "jar": {
+        "maven": [
+            get("MVN_CMD", "mvn"),
+            "compile",
+            "package",
+            "-Dmaven.test.skip=true",
+        ],
+        "gradle": [get("GRADLE_CMD", "%(gradle_cmd)s"), "jar"],
+        "sbt": ["sbt", "stage"],
+    },
+    "android": {"gradle": [get("GRADLE_CMD", "%(gradle_cmd)s"), "compileDebugSources"]},
     "kotlin": {
         "maven": [
             get("MVN_CMD", "mvn"),
@@ -91,7 +107,10 @@ build_tools_map = {
         "yarn": ["yarn", "install"],
         "rush": ["rush", "install", "--bypass-policy", "--no-link"],
     },
-    "go": ["go", "build", "./..."],
+    "go": {
+        "go": ["go", "build", "./..."],
+        "make": ["make", "build"],
+    },
     "php": {
         "init": ["composer", "init", "--quiet"],
         "install": ["composer", "install", "-n", "--ignore-platform-reqs"],
@@ -102,77 +121,104 @@ build_tools_map = {
 }
 
 
+def qwiet_analysis(app_manifest, cwd, env):
+    try:
+        LOG.info(f"Submitting {app_manifest['app']} for Qwiet.AI analysis")
+        build_args = cpg_tools_map["qwiet"]
+        build_args = build_args % dict(**app_manifest)
+        LOG.debug(f"Executing {build_args}")
+        cp = subprocess.run(
+            build_args.split(" "),
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE,
+            cwd=cwd,
+            env=env,
+            check=False,
+            shell=False,
+            encoding="utf-8",
+        )
+        if cp:
+            if LOG.isEnabledFor(DEBUG) and cp.stdout:
+                LOG.debug(cp.stdout)
+            if cp.returncode and cp.stderr:
+                LOG.info(cp.stderr)
+            else:
+                LOG.info(f"{app_manifest['app']} uploaded successfully")
+    except Exception as e:
+        LOG.error(e)
+
+
+def do_x_build(src, env, build_artefacts, tool_lang):
+    for k, v in build_artefacts.items():
+        build_sets = build_tools_map.get(tool_lang)
+        if isinstance(build_sets, dict):
+            build_args = build_tools_map[tool_lang][k]
+        else:
+            build_args = build_sets
+        for afile in v:
+            base_dir = os.path.dirname(afile)
+            build_args_str = " ".join(build_args)
+            if "%(" in build_args_str:
+                gradle_cmd = "gradle"
+                if os.path.exists(os.path.join(base_dir, "gradlew")):
+                    gradle_cmd = "gradlew"
+                    try:
+                        os.chmod(os.path.join(base_dir, "gradlew"), 0o755)
+                    except Exception:
+                        # Ignore errors
+                        pass
+                build_args_str = build_args_str % dict(gradle_cmd=gradle_cmd)
+            try:
+                LOG.debug(
+                    f"Executing {build_args_str} in {base_dir}. To speed up this step, cache your project's dependencies using the CI build cache."
+                )
+                cp = subprocess.run(
+                    build_args_str.split(" "),
+                    stdout=subprocess.PIPE,
+                    stderr=subprocess.PIPE,
+                    cwd=base_dir,
+                    env=env,
+                    check=False,
+                    shell=False,
+                    encoding="utf-8",
+                )
+                if cp and LOG.isEnabledFor(DEBUG) and cp.returncode and cp.stderr:
+                    if cp.stderr:
+                        LOG.debug(cp.stderr)
+            except Exception as e:
+                LOG.debug(e)
+
+
+def do_jar_build(src, env):
+    build_artefacts = {
+        "gradle": find_gradle_files(src),
+        "maven": find_pom_files(src),
+        "sbt": find_sbt_files(src),
+    }
+    settings_xmls = find_files(src, "settings.xml", False, False)
+    if settings_xmls:
+        LOG.info(
+            "Maven settings.xml is present in this repo. This usually means that specific maven settings and profiles needs to be passed via the environment variable MVN_ARGS to build this application correctly."
+        )
+        if not os.getenv("AT_DEBUG_MODE"):
+            LOG.info(
+                "Set the environment variable AT_DEBUG_MODE to debug and look for any build errors"
+            )
+    return do_x_build(src, env, build_artefacts, "jar")
+
+
 def do_go_build(src, env):
-    go_mods = find_go_mods(src)
-    makes = find_makefiles(src)
-    build_args = build_tools_map["go"]
-    for gmod in go_mods:
-        base_dir = os.path.dirname(gmod)
-        try:
-            LOG.debug(f"Executing {' '.join(build_args)} in {base_dir}")
-            subprocess.run(
-                build_args,
-                stdout=subprocess.DEVNULL,
-                stderr=subprocess.DEVNULL,
-                cwd=base_dir,
-                env=env,
-                check=False,
-                shell=False,
-                encoding="utf-8",
-            )
-        except Exception as e:
-            LOG.debug(e)
-    build_args = build_tools_map["make"]
-    for make in makes:
-        base_dir = os.path.dirname(make)
-        try:
-            LOG.debug(f"Executing {' '.join(build_args)} in {base_dir}")
-            subprocess.run(
-                build_args,
-                stdout=subprocess.DEVNULL,
-                stderr=subprocess.DEVNULL,
-                cwd=base_dir,
-                env=env,
-                check=False,
-                shell=False,
-                encoding="utf-8",
-            )
-        except Exception as e:
-            LOG.debug(e)
+    build_artefacts = {"go": find_go_mods(src), "make": find_makefiles(src)}
+    return do_x_build(src, env, build_artefacts, "go")
 
 
 def do_build(tool_lang, src, cwd, env):
-    build_args = None
-    if tool_lang in ("csharp", "scala"):
-        build_args = build_tools_map[tool_lang]
+    if tool_lang in ("csharp",):
+        do_x_build(src, env, {"csharp": find_csharp_artifacts(src)}, "csharp")
+    elif tool_lang in ("jar", "scala"):
+        do_jar_build(src, env)
     elif tool_lang == "go":
         do_go_build(src, env)
-    # For go, we need to detect the go.mod files and attempt build from those directories
-    if build_args:
-        LOG.info(
-            '⚡︎ Attempting to auto build {} "{}"'.format(
-                tool_lang, " ".join(build_args)
-            )
-        )
-        try:
-            cp = subprocess.run(
-                build_args,
-                stdout=subprocess.DEVNULL,
-                stderr=subprocess.DEVNULL,
-                cwd=cwd,
-                env=env,
-                check=False,
-                shell=False,
-                encoding="utf-8",
-            )
-            if cp:
-                if cp.stdout:
-                    LOG.info(cp.stdout)
-                if cp.stderr:
-                    LOG.info(cp.stderr)
-        except Exception as e:
-            LOG.warn("Auto build has failed")
-            LOG.warn(e)
 
 
 def exec_tool(
@@ -183,6 +229,7 @@ def exec_tool(
     joern_home=None,
     use_container=False,
     auto_build=False,
+    extra_args={},
     env=os.environ.copy(),
     stdout=subprocess.DEVNULL,
 ):
@@ -199,10 +246,13 @@ def exec_tool(
         try:
             stderr = subprocess.DEVNULL
             if LOG.isEnabledFor(DEBUG):
-                stderr = subprocess.STDOUT
-            tool_verb = "Building CPG with"
+                stdout = subprocess.PIPE
+                stderr = stdout
+            tool_verb = f"Building CPG with {tool_lang} frontend"
+            if tool_lang == "export":
+                tool_verb = "Exporting CPG with joern-export"
             task = progress.add_task(
-                "[green]" + tool_verb + " " + tool_lang + " frontend",
+                "[green]" + tool_verb,
                 total=100,
                 start=False,
             )
@@ -215,12 +265,12 @@ def exec_tool(
             if "uber_jar" in cmd_with_args:
                 stdout = subprocess.PIPE
                 java_artifacts = find_java_artifacts(src)
-                if len(java_artifacts) == 1:
+                if len(java_artifacts):
                     uber_jar = java_artifacts[0]
             if "csharp_artifacts" in cmd_with_args:
                 stdout = subprocess.PIPE
                 csharp_artifacts = find_csharp_artifacts(src)
-                if len(csharp_artifacts) == 1:
+                if len(csharp_artifacts):
                     csharp_artifacts = csharp_artifacts[0]
             if auto_build:
                 do_build(tool_lang, src, cwd, env)
@@ -242,27 +292,32 @@ def exec_tool(
                     # We need to fix joern_home to the directory inside the container
                     joern_home = ""
                 sbom_cmd_with_args = cpg_tools_map.get("sbom")
-                cpg_out = (
-                    cpg_out_dir
-                    if cpg_out_dir.endswith(".bin.zip")
-                    or cpg_out_dir.endswith(".bin")
-                    or cpg_out_dir.endswith(".cpg")
-                    else os.path.join(
-                        cpg_out_dir,
-                        f"{os.path.basename(amodule)}-{tool_lang}-cpg.bin.zip",
+                sbom_out = ""
+                manifest_out = ""
+                if tool_lang == "export":
+                    cpg_out = cpg_out_dir
+                else:
+                    cpg_out = (
+                        cpg_out_dir
+                        if cpg_out_dir.endswith(".bin.zip")
+                        or cpg_out_dir.endswith(".bin")
+                        or cpg_out_dir.endswith(".cpg")
+                        else os.path.join(
+                            cpg_out_dir,
+                            f"{os.path.basename(amodule)}-{tool_lang}-cpg.bin.zip",
+                        )
                     )
-                )
-                sbom_out = (
-                    cpg_out.replace(".bin.zip", ".bom.xml")
-                    if cpg_out.endswith(".bin.zip")
-                    else f"{cpg_out}.bom.xml"
-                )
-                manifest_out = (
-                    cpg_out.replace(".bin.zip", ".manifest.json")
-                    if cpg_out.endswith(".bin.zip")
-                    else f"{cpg_out}.manifest.json"
-                )
-                LOG.debug(f"CPG file for {tool_lang} is {cpg_out}")
+                    sbom_out = (
+                        cpg_out.replace(".bin.zip", ".bom.xml")
+                        if cpg_out.endswith(".bin.zip")
+                        else f"{cpg_out}.bom.xml"
+                    )
+                    manifest_out = (
+                        cpg_out.replace(".bin.zip", ".manifest.json")
+                        if cpg_out.endswith(".bin.zip")
+                        else f"{cpg_out}.manifest.json"
+                    )
+                    LOG.debug(f"CPG file for {tool_lang} is {cpg_out}")
                 cmd_with_args = cmd_with_args % dict(
                     src=amodule,
                     cpg_out=cpg_out,
@@ -273,11 +328,10 @@ def exec_tool(
                     memory=os.getenv("CPGGEN_MEMORY", max_memory),
                     tool_lang=tool_lang,
                     sbom_out=sbom_out,
+                    **extra_args,
                 )
                 sbom_cmd_with_args = sbom_cmd_with_args % dict(
-                    src=amodule,
-                    tool_lang=tool_lang,
-                    sbom_out=sbom_out,
+                    src=amodule, tool_lang=tool_lang, sbom_out=sbom_out, **extra_args
                 )
                 cmd_list_with_args = cmd_with_args.split(" ")
                 sbom_cmd_list_with_args = sbom_cmd_with_args.split(" ")
@@ -292,6 +346,50 @@ def exec_tool(
                             f"{lang_cmd} is not found. Ensure the PATH variable in your container image is set to the bin directory of Joern."
                         )
                     return
+                # Is this an Export task?
+                if tool_lang == "export":
+                    try:
+                        progress.update(
+                            task, description="Exporting CPG", completed=90, total=100
+                        )
+                        cp = subprocess.run(
+                            cmd_list_with_args,
+                            stdout=stdout,
+                            stderr=stderr,
+                            cwd=cwd,
+                            env=env,
+                            check=False,
+                            shell=False,
+                            encoding="utf-8",
+                        )
+                        if cp and cp.returncode and cp.stderr:
+                            LOG.warn(f"Export CPG has failed for {src}")
+                            if not os.getenv("AT_DEBUG_MODE"):
+                                LOG.info(
+                                    "Set the environment variable AT_DEBUG_MODE to debug to see the debug logs"
+                                )
+                            if cp.stdout:
+                                LOG.info(cp.stdout)
+                            if cp.stderr:
+                                LOG.info("------------------------------")
+                                LOG.info(cp.stderr)
+                                LOG.info("------------------------------")
+                                LOG.info(
+                                    "Please report the above error to https://github.com/joernio/joern/issues"
+                                )
+                        else:
+                            if os.path.exists(src):
+                                LOG.info(
+                                    f"CPG {src} successfully exported to {cpg_out_dir}"
+                                )
+                            else:
+                                LOG.warn(
+                                    f"Unable to export {src} to {cpg_out_dir}. Try running joern-export manually using the command {' '.join(cmd_list_with_args)}"
+                                )
+                    except Exception:
+                        LOG.warn(f"Unable to export {src} to {cpg_out_dir}")
+                    progress.update(task, completed=100, total=100)
+                    continue
                 LOG.debug(
                     '⚡︎ Generating CPG for the {} app "{}" - "{}"'.format(
                         tool_lang,
@@ -304,6 +402,12 @@ def exec_tool(
                     cwd = os.getcwd()
                 # Generate sbom first since this would even download dependencies for java
                 try:
+                    progress.update(
+                        task,
+                        description="Generating SBoM using cdxgen",
+                        completed=10,
+                        total=100,
+                    )
                     subprocess.run(
                         sbom_cmd_list_with_args,
                         stdout=stdout,
@@ -317,6 +421,9 @@ def exec_tool(
                 except Exception:
                     # Ignore sbom generation errors
                     pass
+                progress.update(
+                    task, description="Generating CPG", completed=20, total=100
+                )
                 cp = subprocess.run(
                     cmd_list_with_args,
                     stdout=stdout,
@@ -330,14 +437,11 @@ def exec_tool(
                 if cp and stdout == subprocess.PIPE:
                     for line in cp.stdout:
                         progress.update(task, completed=5)
-                if (
-                    cp
-                    and LOG.isEnabledFor(DEBUG)
-                    and cp.returncode
-                    and cp.stdout is not None
-                ):
-                    LOG.debug(cp.stdout)
-                progress.update(task, completed=100, total=100)
+                if cp and LOG.isEnabledFor(DEBUG) and cp.returncode:
+                    if cp.stdout:
+                        LOG.debug(cp.stdout)
+                    if cp.stderr:
+                        LOG.debug(cp.stderr)
                 if os.path.exists(cpg_out):
                     # go2cpg seems to produce a cpg without read permissions
                     try:
@@ -371,26 +475,39 @@ def exec_tool(
                             app_base_name = os.getenv("GITHUB_REPOSITORY").split("/")[
                                 -1
                             ]
-                        json.dump(
-                            {
-                                "src": amodule,
-                                "group": app_base_name,
-                                "app": f"{app_base_name}-{language}",
-                                "cpg": cpg_out,
-                                "sbom": sbom_out,
-                                "language": language,
-                                "cpg_frontend_invocation": " ".join(cmd_list_with_args),
-                                "sbom_invocation": " ".join(sbom_cmd_list_with_args),
-                            },
-                            mfp,
-                        )
+                        app_manifest = {
+                            "src": amodule,
+                            "group": app_base_name,
+                            "app": f"{app_base_name}-{language}",
+                            "cpg": cpg_out,
+                            "sbom": sbom_out,
+                            "language": language,
+                            "cpg_frontend_invocation": " ".join(cmd_list_with_args),
+                            "sbom_invocation": " ".join(sbom_cmd_list_with_args),
+                        }
+                        if os.getenv("SHIFTLEFT_ACCESS_TOKEN"):
+                            progress.update(
+                                task,
+                                description="Uploading to Qwiet AI for analysis",
+                                completed=90,
+                                total=100,
+                            )
+                            qwiet_analysis(app_manifest, cwd, env)
+                        json.dump(app_manifest, mfp)
                 else:
                     LOG.info(f"CPG {cpg_out} was not generated for {tool_lang}")
+                    if not os.getenv("AT_DEBUG_MODE"):
+                        LOG.info(
+                            "Set the environment variable AT_DEBUG_MODE to debug to see the debug logs"
+                        )
                     if cp.stdout:
                         LOG.info(cp.stdout)
                     if cp.stderr:
                         LOG.info(cp.stderr)
+                progress.update(task, completed=100, total=100)
         except Exception as e:
-            if task:
-                progress.update(task, completed=20, total=10, visible=False)
+            if not os.getenv("AT_DEBUG_MODE"):
+                LOG.info(
+                    "Set the environment variable AT_DEBUG_MODE to debug to see the debug logs"
+                )
             LOG.error(e)
