@@ -34,6 +34,10 @@ ignore_directories = [
     "mock",
     "mocks",
     "vendor",
+    "instancemode-tests",
+    "integration-tests",
+    "oauth2-tests",
+    "twofactor-tests",
 ]
 
 ignore_files = [
@@ -123,12 +127,16 @@ def is_ignored_dir(base_dir, dir_name):
     dir_name = dir_name.lower()
     if dir_name.startswith("."):
         return True
-    elif dir_name.startswith("/" + base_dir):
-        dir_name = re.sub(r"^/" + base_dir + "/", "", dir_name)
+    elif dir_name.startswith(os.path.sep + base_dir):
+        dir_name = re.sub(r"^" + os.path.sep + base_dir + os.path.sep, "", dir_name)
     elif dir_name.startswith(base_dir):
-        dir_name = re.sub(r"^" + base_dir + "/", "", dir_name)
+        dir_name = re.sub(r"^" + base_dir + os.path.sep, "", dir_name)
     for d in ignore_directories:
-        if dir_name == d or dir_name.startswith(d) or ("/" + d + "/") in dir_name:
+        if (
+            dir_name == d
+            or dir_name.startswith(d)
+            or (os.path.sep + d + os.path.sep) in dir_name
+        ):
             return True
     return False
 
@@ -200,6 +208,18 @@ def find_makefiles(search_dir):
     return find_files(search_dir, "Makefile", False, False)
 
 
+def find_gradle_files(search_dir):
+    return find_files(search_dir, "build.gradle", False, False)
+
+
+def find_pom_files(search_dir):
+    return find_files(search_dir, "pom.xml", False, False)
+
+
+def find_sbt_files(search_dir):
+    return find_files(search_dir, "build.sbt", False, False)
+
+
 def check_command(cmd):
     """
     Method to check if command is available
